var express = require('express');
var http = require('http');
var app = express();
var server = http.createServer(app);
var port = process.env.PORT || 8080;
var io = require('socket.io')(server);

// list of zone IDs that have been mapped with locations
var mappedZones = ['w', 'k1', 'k2'];

/*************************************** handle express logic ***************************************/
// send index on request for '/'
app.get('/', function (req, res) {
    res.sendFile(__dirname + '/public/index.html');
});

// send public files on request
app.use('/public', express.static(__dirname + '/public'));

// start the server
server.listen(port, function () {
    console.log('Started server on ' + port);
});


/*************************************** initialize data ***************************************/
/**
 * Note: In a real implementation, this data would reside in a relational database, having two
 * tables: ZoneData and LocationData.  The zoneId fields in LocationData would act as foreign keys,
 * allowing one to ascertain the zone a location is in and obtain its price etc.  However, since we
 * are limited by both time and hosting constraints, the data will be store in the following objects.
 */
var zoneData = {
    'w': {
        name: 'Waterloo',
        capacity: 100,
        currentOccupancy: 87,
        band1Price: 2.00,
        band6Price: 3.00,
        occupancyMultiplier: 1.2
    },
    'k1': {
        name: 'Kennington',
        capacity: 200,
        currentOccupancy: 87,
        band1Price: 2.00,
        band6Price: 3.00,
        occupancyMultiplier: 1.2
    },
    'k2': {
        name: 'Kennington',
        capacity: 60,
        currentOccupancy: 27,
        band1Price: 2.00,
        band6Price: 3.00,
        occupancyMultiplier: 1.2
    },
    's': {
        name: 'Stockwell',
        capacity: 80,
        currentOccupancy: 10,
        band1Price: 2.00,
        band6Price: 3.00,
        occupancyMultiplier: 1.2
    },
    'l1': {
        name: 'Clapham',
        capacity: 150,
        currentOccupancy: 68,
        band1Price: 2.00,
        band6Price: 3.00,
        occupancyMultiplier: 1.2
    },
    'l2': {
        name: 'Clapham',
        capacity: 90,
        currentOccupancy: 20,
        band1Price: 2.00,
        band6Price: 3.00,
        occupancyMultiplier: 1.2
    },
    'l3': {
        name: 'Clapham',
        capacity: 80,
        currentOccupancy: 60,
        band1Price: 2.00,
        band6Price: 3.00,
        occupancyMultiplier: 1.2
    },
    'c1': {
        name: 'Clapham',
        capacity: 130,
        currentOccupancy: 48,
        band1Price: 2.00,
        band6Price: 3.00,
        occupancyMultiplier: 1.2
    },
    'c2': {
        name: 'Clapham',
        capacity: 120,
        currentOccupancy: 66,
        band1Price: 2.00,
        band6Price: 3.00,
        occupancyMultiplier: 1.2
    },
    'c3': {
        name: 'Clapham',
        capacity: 50,
        currentOccupancy: 43,
        band1Price: 2.00,
        band6Price: 3.00,
        occupancyMultiplier: 1.2
    },
    'c4': {
        name: 'Clapham',
        capacity: 63,
        currentOccupancy: 47,
        band1Price: 2.00,
        band6Price: 3.00,
        occupancyMultiplier: 1.2
    },
    'c5': {
        name: 'Clapham',
        capacity: 33,
        currentOccupancy: 14,
        band1Price: 2.00,
        band6Price: 3.00,
        occupancyMultiplier: 1.2
    },
    'b1': {
        name: 'Brixton',
        capacity: 200,
        currentOccupancy: 147,
        band1Price: 2.00,
        band6Price: 3.00,
        occupancyMultiplier: 1.2
    },
    'b2': {
        name: 'Brixton',
        capacity: 210,
        currentOccupancy: 198,
        band1Price: 2.00,
        band6Price: 3.00,
        occupancyMultiplier: 1.2
    },
    'b3': {
        name: 'Brixton',
        capacity: 100,
        currentOccupancy: 99,
        band1Price: 2.00,
        band6Price: 3.00,
        occupancyMultiplier: 1.2
    },
    'e': {
        name: 'Brixton',
        capacity: 120,
        currentOccupancy: 67,
        band1Price: 2.00,
        band6Price: 3.00,
        occupancyMultiplier: 1.4
    },
    'n1': {
        name: 'Herne Hill',
        capacity: 100,
        currentOccupancy: 37,
        band1Price: 2.00,
        band6Price: 3.00,
        occupancyMultiplier: 1.3
    },
    'n2': {
        name: 'Herne Hill',
        capacity: 90,
        currentOccupancy: 67,
        band1Price: 2.00,
        band6Price: 3.00,
        occupancyMultiplier: 1.3
    },
    'a': {
        name: 'Camberwell',
        capacity: 60,
        currentOccupancy: 20,
        band1Price: 2.00,
        band6Price: 3.00,
        occupancyMultiplier: 1.3
    },
    'r': {
        name: 'Thornton',
        capacity: 83,
        currentOccupancy: 46,
        band1Price: 2.00,
        band6Price: 3.00,
        occupancyMultiplier: 1.1
    },
    'q1': {
        name: 'Brixton Hill East',
        capacity: 123,
        currentOccupancy: 93,
        band1Price: 2.00,
        band6Price: 3.00,
        occupancyMultiplier: 1.1
    },
    'q2': {
        name: 'Brixton Hill East',
        capacity: 105,
        currentOccupancy: 33,
        band1Price: 2.00,
        band6Price: 3.00,
        occupancyMultiplier: 1.1
    },
    'h1': {
        name: 'Tulse Hill',
        capacity: 73,
        currentOccupancy: 63,
        band1Price: 2.00,
        band6Price: 3.00,
        occupancyMultiplier: 1.1
    },
    'h2': {
        name: 'Tulse Hill',
        capacity: 50,
        currentOccupancy: 23,
        band1Price: 2.00,
        band6Price: 3.00,
        occupancyMultiplier: 1.1
    }
};

var locationData = {
    83880: {
        name: 'Upper Marsh',
        zoneId: 'w',
        capacity: 12,
        currentOccupancy: 0
    },
    83881: {
        name: 'Lower Marsh',
        zoneId: 'w',
        capacity: 23,
        currentOccupancy: 0
    },
    83883: {
        name: 'King Edward Walk',
        zoneId: 'w',
        capacity: 27,
        currentOccupancy: 0
    },
    83884: {
        name: 'Cosser Street',
        zoneId: 'w',
        capacity: 28,
        currentOccupancy: 0
    },
    83885: {
        name: 'Hercules Road',
        zoneId: 'w',
        capacity: 23,
        currentOccupancy: 0
    },
    83886: {
        name: 'Hercules Road',
        zoneId: 'w',
        capacity: 15,
        currentOccupancy: 0
    },
    83887: {
        name: 'Hercules Road',
        zoneId: 'w',
        capacity: 12,
        currentOccupancy: 0
    },
    83888: {
        name: 'Hercules Road',
        zoneId: 'w',
        capacity: 23,
        currentOccupancy: 0
    },
    83889: {
        name: 'Centaur Street',
        zoneId: 'w',
        capacity: 11,
        currentOccupancy: 0
    },
    83890: {
        name: 'Royal Street',
        zoneId: 'w',
        capacity: 20,
        currentOccupancy: 0
    },
    83891: {
        name: 'Royal Stret',
        zoneId: 'w',
        capacity: 29,
        currentOccupancy: 0
    },
    83892: {
        name: 'Frazier Street',
        zoneId: 'w',
        capacity: 16,
        currentOccupancy: 0
    },
    83893: {
        name: 'Coral Street',
        zoneId: 'w',
        capacity: 15,
        currentOccupancy: 0
    },
    83895: {
        name: 'Webber Street',
        zoneId: 'w',
        capacity: 24,
        currentOccupancy: 0
    },
    83896: {
        name: 'Webber Street',
        zoneId: 'w',
        capacity: 25,
        currentOccupancy: 0
    },
    83897: {
        name: 'The Cut',
        zoneId: 'w',
        capacity: 14,
        currentOccupancy: 0
    },
    83898: {
        name: 'The Cut',
        zoneId: 'w',
        capacity: 21,
        currentOccupancy: 0
    },
    83899: {
        name: 'The Cut',
        zoneId: 'w',
        capacity: 21,
        currentOccupancy: 0
    },
    83900: {
        name: 'Short Street',
        zoneId: 'w',
        capacity: 28,
        currentOccupancy: 0
    },
    83901: {
        name: 'Cons Street',
        zoneId: 'w',
        capacity: 23,
        currentOccupancy: 0
    },
    83902: {
        name: 'Greet Street',
        zoneId: 'w',
        capacity: 19,
        currentOccupancy: 0
    },
    83903: {
        name: 'Wooton Street',
        zoneId: 'w',
        capacity: 15,
        currentOccupancy: 0
    },
    83904: {
        name: 'Roupell Street',
        zoneId: 'w',
        capacity: 18,
        currentOccupancy: 0
    },
    83905: {
        name: 'Whittlesey Street',
        zoneId: 'w',
        capacity: 20,
        currentOccupancy: 0
    },
    83906: {
        name: 'Exton Street',
        zoneId: 'w',
        capacity: 15,
        currentOccupancy: 0
    },
    83907: {
        name: 'Secker Street',
        zoneId: 'w',
        capacity: 16,
        currentOccupancy: 0
    },
    83908: {
        name: 'Coin Street',
        zoneId: 'w',
        capacity: 23,
        currentOccupancy: 0
    },
    83909: {
        name: 'Ducky Street',
        zoneId: 'w',
        capacity: 20,
        currentOccupancy: 0
    },
    83910: {
        name: 'Cornwall Road',
        zoneId: 'w',
        capacity: 12,
        currentOccupancy: 0
    },
    83911: {
        name: 'Doon Street',
        zoneId: 'w',
        capacity: 27,
        currentOccupancy: 0
    },
    83912: {
        name: 'Upper Ground',
        zoneId: 'w',
        capacity: 21,
        currentOccupancy: 0
    },
    83915: {
        name: 'Concert Hall Approach',
        zoneId: 'w',
        capacity: 23,
        currentOccupancy: 0
    },
    83916: {
        name: 'Chicheley Street',
        zoneId: 'w',
        capacity: 22,
        currentOccupancy: 0
    },
    83917: {
        name: 'Hatfields',
        zoneId: 'w',
        capacity: 20,
        currentOccupancy: 0
    },
    83918: {
        name: 'Cornwall Road',
        zoneId: 'w',
        capacity: 18,
        currentOccupancy: 0
    },

    83567: {
        name: 'Sail Street',
        zoneId: 'k1',
        capacity: 14,
        currentOccupancy: 0
    },
    83568: {
        name: 'Pratt Walk',
        zoneId: 'k1',
        capacity: 25,
        currentOccupancy: 0
    },
    83570: {
        name: 'Juxon Street',
        zoneId: 'k1',
        capacity: 13,
        currentOccupancy: 0
    },
    83569: {
        name: 'Randall Road',
        zoneId: 'k1',
        capacity: 11,
        currentOccupancy: 0
    },
    83571: {
        name: 'Lambeth Walk',
        zoneId: 'k1',
        capacity: 29,
        currentOccupancy: 0
    },
    83572: {
        name: 'Lambeth Walk',
        zoneId: 'k1',
        capacity: 20,
        currentOccupancy: 0
    },
    83574: {
        name: 'Lambeth High Street',
        zoneId: 'k1',
        capacity: 17,
        currentOccupancy: 0
    },
    83575: {
        name: 'Black Prince Road',
        zoneId: 'k1',
        capacity: 24,
        currentOccupancy: 0
    },
    83577: {
        name: 'Vauxhall Street',
        zoneId: 'k1',
        capacity: 10,
        currentOccupancy: 0
    },
    83578: {
        name: 'Goding Street',
        zoneId: 'k1',
        capacity: 12,
        currentOccupancy: 0
    },
    83579: {
        name: 'Glasshouse Street',
        zoneId: 'k1',
        capacity: 10,
        currentOccupancy: 0
    },
    83580: {
        name: 'Vauxhall Walk',
        zoneId: 'k1',
        capacity: 28,
        currentOccupancy: 0
    },
    83581: {
        name: 'Vauxhall Walk',
        zoneId: 'k1',
        capacity: 21,
        currentOccupancy: 0
    },
    83582: {
        name: 'Sancroft Street',
        zoneId: 'k1',
        capacity: 11,
        currentOccupancy: 0
    },
    83583: {
        name: 'Tyers Street',
        zoneId: 'k1',
        capacity: 15,
        currentOccupancy: 0
    },
    83584: {
        name: 'Tyers Street',
        zoneId: 'k1',
        capacity: 28,
        currentOccupancy: 0
    },
    83585: {
        name: 'Cardigan Street',
        zoneId: 'k1',
        capacity: 12,
        currentOccupancy: 0
    },
    83587: {
        name: 'Montford Place',
        zoneId: 'k1',
        capacity: 18,
        currentOccupancy: 0
    },
    83588: {
        name: 'Montford Place',
        zoneId: 'k2',
        capacity: 23,
        currentOccupancy: 0
    },
    83589: {
        name: 'Kennington Road',
        zoneId: 'k2',
        capacity: 20,
        currentOccupancy: 0
    },
    83590: {
        name: 'Vauxhall Grove',
        zoneId: 'k1',
        capacity: 13,
        currentOccupancy: 0
    },
    83591: {
        name: 'Langley Lane',
        zoneId: 'k1',
        capacity: 21,
        currentOccupancy: 0
    },
    82593: {
        name: 'Clayton Street',
        zoneId: 'k1',
        capacity: 27,
        currentOccupancy: 0
    },
    83594: {
        name: 'Bowling Green Street',
        zoneId: 'k1',
        capacity: 14,
        currentOccupancy: 0
    },
    83595: {
        name: 'Magee Street',
        zoneId: 'k1',
        capacity: 25,
        currentOccupancy: 0
    },
    83596: {
        name: 'Kennington Oval',
        zoneId: 'k1',
        capacity: 12,
        currentOccupancy: 0
    },
    83597: {
        name: 'Kennington Oval',
        zoneId: 'k1',
        capacity: 21,
        currentOccupancy: 0
    },
    83599: {
        name: 'Hanover Gardens',
        zoneId: 'k1',
        capacity: 27,
        currentOccupancy: 0
    },
    83600: {
        name: 'Elias Place',
        zoneId: 'k1',
        capacity: 28,
        currentOccupancy: 0
    },
    83601: {
        name: 'Claylands Road',
        zoneId: 'k1',
        capacity: 26,
        currentOccupancy: 0
    },
    83602: {
        name: 'Prima Road',
        zoneId: 'k1',
        capacity: 26,
        currentOccupancy: 0
    },
    83603: {
        name: 'Offley Road',
        zoneId: 'k1',
        capacity: 26,
        currentOccupancy: 0
    },
    83604: {
        name: 'Handforth Road',
        zoneId: 'k1',
        capacity: 15,
        currentOccupancy: 0
    },
    83605: {
        name: 'Crewdson Road',
        zoneId: 'k1',
        capacity: 24,
        currentOccupancy: 0
    },
    83606: {
        name: 'Walcot Square',
        zoneId: 'k1',
        capacity: 20,
        currentOccupancy: 0
    },
    83607: {
        name: 'Bishop\'s Terrace',
        zoneId: 'k1',
        capacity: 14,
        currentOccupancy: 0
    },
    83608: {
        name: 'Wincott Street',
        zoneId: 'k1',
        capacity: 22,
        currentOccupancy: 0
    },
    83609: {
        name: 'Renfrew Road',
        zoneId: 'k1',
        capacity: 14,
        currentOccupancy: 0
    },
    83610: {
        name: 'Denny Street',
        zoneId: 'k1',
        capacity: 18,
        currentOccupancy: 0
    },
    83611: {
        name: 'Cleaver Square',
        zoneId: 'k2',
        capacity: 17,
        currentOccupancy: 0
    },
    83612: {
        name: 'Cleaver Street',
        zoneId: 'k2',
        capacity: 16,
        currentOccupancy: 0
    },
    83613: {
        name: 'Radcor Street',
        zoneId: 'k2',
        capacity: 16,
        currentOccupancy: 0
    },
    83614: {
        name: 'Milverton Street',
        zoneId: 'k2',
        capacity: 29,
        currentOccupancy: 0
    },
    83615: {
        name: 'Stannary Street',
        zoneId: 'k2',
        capacity: 10,
        currentOccupancy: 0
    },
    83616: {
        name: 'Meadow Road',
        zoneId: 'k1',
        capacity: 26,
        currentOccupancy: 0
    },
    83619: {
        name: 'Kennings Way',
        zoneId: 'k2',
        capacity: 19,
        currentOccupancy: 0
    },
    83620: {
        name: 'Chester Way',
        zoneId: 'k1',
        capacity: 27,
        currentOccupancy: 0
    },
    83621: {
        name: 'Fentiman Road',
        zoneId: 'k1',
        capacity: 11,
        currentOccupancy: 0
    },
    83622: {
        name: 'Fentiman Road',
        zoneId: 'k1',
        capacity: 26,
        currentOccupancy: 0
    },
    83623: {
        name: 'Fentiman Road',
        zoneId: 'k1',
        capacity: 15,
        currentOccupancy: 0
    },
    83624: {
        name: 'Fentiman Road',
        zoneId: 'k1',
        capacity: 21,
        currentOccupancy: 0
    },
    83625: {
        name: 'Fentiman Road',
        zoneId: 'k1',
        capacity: 30,
        currentOccupancy: 0
    },
    83626: {
        name: 'Ashmole Street',
        zoneId: 'k1',
        capacity: 27,
        currentOccupancy: 0
    },
    83628: {
        name: 'Palfrey',
        zoneId: 'k1',
        capacity: 13,
        currentOccupancy: 0
    },
    83629: {
        name: 'Richborne Terrace',
        zoneId: 'k1',
        capacity: 30,
        currentOccupancy: 0
    },
    83630: {
        name: 'Richborne Terrace',
        zoneId: 'k1',
        capacity: 25,
        currentOccupancy: 0
    },
    83631: {
        name: 'Carroun Road',
        zoneId: 'k1',
        capacity: 30,
        currentOccupancy: 0
    },
    83632: {
        name: 'Carroun Road',
        zoneId: 'k1',
        capacity: 30,
        currentOccupancy: 0
    },
    83635: {
        name: 'Rita Road',
        zoneId: 'k1',
        capacity: 10,
        currentOccupancy: 0
    },
    83636: {
        name: 'Rita Road',
        zoneId: 'k1',
        capacity: 17,
        currentOccupancy: 0
    },
    83643: {
        name: 'Old South Lambeth Road',
        zoneId: 'k1',
        capacity: 26,
        currentOccupancy: 0
    },
    83644: {
        name: 'Heyford Avenue',
        zoneId: 'k1',
        capacity: 16,
        currentOccupancy: 0
    },
    83645: {
        name: 'Heyford Avenue',
        zoneId: 'k1',
        capacity: 15,
        currentOccupancy: 0
    },
    83647: {
        name: 'Meadow Place',
        zoneId: 'k1',
        capacity: 26,
        currentOccupancy: 0
    },
    83648: {
        name: 'South Island Place',
        zoneId: 'k1',
        capacity: 21,
        currentOccupancy: 0
    },
    83649: {
        name: 'Mowll Street',
        zoneId: 'k1',
        capacity: 14,
        currentOccupancy: 0
    },
    83650: {
        name: 'Caldwell Street',
        zoneId: 'k1',
        capacity: 15,
        currentOccupancy: 0
    },
    83651: {
        name: 'Caldwell Street',
        zoneId: 'k1',
        capacity: 29,
        currentOccupancy: 0
    },
    83653: {
        name: 'Hackford Road',
        zoneId: 'k1',
        capacity: 13,
        currentOccupancy: 0
    },
    83654: {
        name: 'Hackford Road',
        zoneId: 'k1',
        capacity: 22,
        currentOccupancy: 0
    },
    83656: {
        name: 'Southey Road',
        zoneId: 'k1',
        capacity: 25,
        currentOccupancy: 0
    },
    83657: {
        name: 'Cranworth Gardens',
        zoneId: 'k1',
        capacity: 12,
        currentOccupancy: 0
    },
    83658: {
        name: 'Cranworth Gardens',
        zoneId: 'k1',
        capacity: 26,
        currentOccupancy: 0
    },
    83655: {
        name: 'Hackford Road',
        zoneId: 'k1',
        capacity: 23,
        currentOccupancy: 0
    },
    83659: {
        name: 'Hillard Street',
        zoneId: 'k1',
        capacity: 26,
        currentOccupancy: 0
    },
    83660: {
        name: 'Hillard Street',
        zoneId: 'k1',
        capacity: 15,
        currentOccupancy: 0
    },
    83662: {
        name: 'Bondway',
        zoneId: 'k1',
        capacity: 13,
        currentOccupancy: 0
    },
    83663: {
        name: 'Miles Street',
        zoneId: 'k1',
        capacity: 24,
        currentOccupancy: 0
    },
    83664: {
        name: 'Miles Street',
        zoneId: 'k1',
        capacity: 24,
        currentOccupancy: 0
    },
    83666: {
        name: 'Cobbet Street',
        zoneId: 'k1',
        capacity: 24,
        currentOccupancy: 0
    },
    83667: {
        name: 'Kibworth Street',
        zoneId: 'k1',
        capacity: 10,
        currentOccupancy: 0
    },
    83668: {
        name: 'Liberty Street',
        zoneId: 'k1',
        capacity: 14,
        currentOccupancy: 0
    },
    83669: {
        name: 'Liberty Street',
        zoneId: 'k1',
        capacity: 25,
        currentOccupancy: 0
    },
    83670: {
        name: 'Liberty Street',
        zoneId: 'k1',
        capacity: 27,
        currentOccupancy: 0
    },
    83671: {
        name: 'Morat Street',
        zoneId: 'k1',
        capacity: 15,
        currentOccupancy: 0
    },
    83673: {
        name: 'Durand Gardens',
        zoneId: 'k1',
        capacity: 27,
        currentOccupancy: 0
    },
    83674: {
        name: 'Durand Gardens',
        zoneId: 'k1',
        capacity: 28,
        currentOccupancy: 0
    },
    83637: {
        name: 'Dorset Road',
        zoneId: 'k1',
        capacity: 14,
        currentOccupancy: 0
    },
    83638: {
        name: 'Dorset Road',
        zoneId: 'k1',
        capacity: 19,
        currentOccupancy: 0
    },
    83642: {
        name: 'Dorset Road',
        zoneId: 'k1',
        capacity: 30,
        currentOccupancy: 0
    }
};

var MODE_CAPACITY = 1;
var MODE_OCCUPANCY = 2;

/**
 * Returns a random integer within a range.
 */
function randomInt(min, max) {
    return Math.floor(Math.random() * (max - min + 1)) + min;
}

/**
 * Finds valid locations to edit.
 * @param zoneId the zone ID of the locations to find.
 * @param mode The mode, either MODE_CAPACITY or MODE_OCCUPANCY
 * @param increment The increase or decrease in capacity / occupancy.
 * @returns {Array} An array of valid locations to edit.
 */
function findLocations(zoneId, mode, increment) {
    var optimalLocations = [];
    var validLocations = [];
    for (var locationId in locationData) {
        if (locationData.hasOwnProperty(locationId) && locationData[locationId].zoneId == zoneId) {
            var location = locationData[locationId];
            if ((mode == MODE_CAPACITY && increment < 0 && location.capacity > 1) ||
                (mode == MODE_OCCUPANCY && increment > 0)) {
                // in these two cases, we build up 'acceptable' locations to return
                validLocations.push(location);
            }
            // the following if statement can definitely condensed; this may be easier to read
            if ((mode == MODE_CAPACITY && increment > 0) ||
                (mode == MODE_CAPACITY && increment < 0 && location.currentOccupancy < location.capacity && location.capacity > 1) ||
                (mode == MODE_OCCUPANCY && increment > 0 && location.currentOccupancy < location.capacity) ||
                (mode == MODE_OCCUPANCY && increment < 0 && location.currentOccupancy > 0)) {
                optimalLocations.push(location);
            }
        }
    }
    // determine which array to return for the two special cases
    if (!optimalLocations.length && ((mode == MODE_CAPACITY && increment < 0) ||
        (mode == MODE_OCCUPANCY && increment > 0))) {
        return validLocations;
    }
    return optimalLocations;
}

/**
 * Randomizes the occupancy or capacity of a zone.
 * @param zoneId The zone to edit.
 * @param mode The mode, either MODE_CAPACITY or MODE_OCCUPANCY.
 * @param difference the amount to increment the capacity or occupancy
 * by (can be negative).
 */
function editZone(zoneId, mode, difference) {
    if (!difference) {
        return;
    }
    var increment = difference > 0 ? 1 : -1;
    var locations = findLocations(zoneId, mode, increment);
    for (var i = 0; i < Math.abs(difference) && locations.length; i++) {
        // choose a random location from the set
        var index = Math.floor(Math.random() * locations.length);
        var location = locations[index];
        if (mode == MODE_CAPACITY) {
            location.capacity += increment;
        } else if (mode == MODE_OCCUPANCY) {
            location.currentOccupancy += increment;
        }

        // remove a location if it becomes full, empty, or has a capacity of 1 depending on mode
        if ((mode == MODE_CAPACITY && increment < 0 && (location.currentOccupancy >= location.capacity || location.capacity <= 1)) ||
            (mode == MODE_OCCUPANCY && increment < 0 && location.currentOccupancy <= 0) ||
            (mode == MODE_OCCUPANCY && increment > 0 && location.currentOccupancy >= location.capacity)) {
            locations.splice(index, 1);
        }

        // if the array of locations is now empty, get an updated copy
        if (!locations.length) {
            locations = findLocations(zoneId, mode, increment);
        }
    }
}

/**
 * Randomizes the occupancies of each parking location in the locationData object.
 */
function randomizeLocations() {
    for (var locationId in locationData) {
        if (locationData.hasOwnProperty(locationId)) {
            locationData[locationId].capacity = randomInt(15, 30);
            locationData[locationId].currentOccupancy = randomInt(0, locationData[locationId].capacity);
        }
    }
}

/**
 * Sets the zone capacities and occupancies based on the sum of location data.
 * This only applies to zones w and k, as these are the only zones which have
 * been mapped.
 */
function setZoneData() {
    for (var zoneId in zoneData) {
        if (zoneData.hasOwnProperty(zoneId) && (mappedZones.indexOf(zoneId) > -1)) {
            zoneData[zoneId].capacity = 0;
            zoneData[zoneId].currentOccupancy = 0;
        }
    }
    for (var locationId in locationData) {
        if (locationData.hasOwnProperty(locationId)) {
            var zone = zoneData[locationData[locationId].zoneId];
            zone.capacity += locationData[locationId].capacity;
            zone.currentOccupancy += locationData[locationId].currentOccupancy;
        }
    }
}


randomizeLocations();
setZoneData();
/*************************************** handle socket.io logic ***************************************/
/**
 * As you can see, th server-side logic is extremely simple.  It's basically just a set of data queries.
 * In reality, we would just query the database first instead of just returning the raw values.
 * Note: For this example, teh entire location and zone objects are passed during each transaction,
 * even when originating from a clinet.  In reality, this should be avoided, as it allows any client
 * to access and alter all of the available data.  Simply providing a limited set of actions for
 * clients accomplishes this task easily.
 */

io.on('connection', function (clientSocket) {

    // handle a request for the currentOccupancy parking data
    clientSocket.on('get-data', function () {
        clientSocket.emit('server-update', {
            zoneData: zoneData,
            locationData: locationData
        });
    });

    // handle a request to alter the entire zone data set
    clientSocket.on('set-data', function (data) {
        // alter the zone data to match the new values
        zoneData = data.zoneData;
        locationData = data.locationData;
        setZoneData();

        // emit changes to all listeners
        io.emit('server-update', {
            zoneData: zoneData,
            locationData: locationData
        });
    });


    clientSocket.on('start-parking', function (data) {
        var location = locationData[data.locationId];
        if (!location) {
            console.log('Bad location for start parking');
            return;
        }
        location.currentOccupancy++;
        setZoneData();
        // emit changes to all listeners
        io.emit('server-update', {
            zoneData: zoneData,
            locationData: locationData
        });
    });

    clientSocket.on('stop-parking', function (data) {
        var location = locationData[data.locationId];
        if (!location) {
            console.log('Bad location for stop parking');
            return;
        }
        location.currentOccupancy--;
        setZoneData();
        // emit changes to all listeners
        io.emit('server-update', {
            zoneData: zoneData,
            locationData: locationData
        });
    });

    // handle randomization in edit mode
    clientSocket.on('random', function () {
        randomizeLocations();
        setZoneData();
        // emit changes to all listeners
        io.emit('server-update', {
            zoneData: zoneData,
            locationData: locationData
        });
    });

    // handle alterations of zone capacity
    clientSocket.on('edit-capacity', function (data) {
        if (data.zoneId && data.difference && zoneData[data.zoneId]) {
            if (mappedZones.indexOf(data.zoneId) > -1) {
                editZone(data.zoneId, MODE_CAPACITY, data.difference);
            } else {
                // if the zone isn't mapped, directly change its capacity
                zoneData[data.zoneId].capacity += data.difference;
            }
            setZoneData();
        }
        io.emit('server-update', {
            zoneData: zoneData,
            locationData: locationData
        });
    });

    // handle alterations of zone occupancy
    clientSocket.on('edit-occupancy', function (data) {
        if (data.zoneId && data.difference && zoneData[data.zoneId]) {
            if (mappedZones.indexOf(data.zoneId) > -1) {
                editZone(data.zoneId, MODE_OCCUPANCY, data.difference);
            } else {
                // if the zone isn't mapped, directly change its occupancy
                zoneData[data.zoneId].currentOccupancy += data.difference;
            }
            setZoneData();
        }
        io.emit('server-update', {
            zoneData: zoneData,
            locationData: locationData
        });
    });
});