/**
 * Created by Connor Weeks on 3/18/2017.
 */

function round(number, decimalPlaces) {
    var conversion = Math.pow(10, decimalPlaces);
    return Math.round(number * conversion) / conversion;
}

function displayData(id, type, forceFormatting) {
    var data = getData(id, type);
    if (data) {
        // determine whether or not the data has location values
        var isLocation = data.locationId != 'None';
        $('#zone-label').text('Zone: ' + data.zoneName);
        $('#location-label').text('Location: ' + data.locationName);
        $('#occupancy-label').text('Occupancy: ' + (data.fractionOccupied * 100).toFixed() + '%');
        $('#capacity').val(data.capacity).prop('disabled', !isLocation);
        var band1Price = $('#band1-price');
        band1Price.prop('disabled', isLocation);
        if (forceFormatting || (parseFloat(band1Price.val()) != data.band1Price)) {
            band1Price.val(data.band1Price.toFixed(2))
        }
        var band6Price = $('#band6-price');
        band6Price.prop('disabled', isLocation);
        if (forceFormatting || (parseFloat(band6Price.val()) != data.band6Price)) {
            band6Price.val(data.band6Price.toFixed(2))
        }
        var occupancyMultiplier = $('#occupancy-multiplier');
        occupancyMultiplier.prop('disabled', isLocation);
        // only reset the value if it is numerically different (since '1.' and '1' are the same)
        if (forceFormatting || (parseFloat(occupancyMultiplier.val()) != data.occupancyMultiplier)) {
            console.log('nope');
            occupancyMultiplier.val(data.occupancyMultiplier.toFixed(2));
        }
    } else {
        $('#zone-label').text('Zone: None');
        $('#location-label').text('Location: None');
        $('#occupancy-label').text('Occupancy: None');
        $('#capacity').val('');

        $('#band1-price').val('');
        $('#band6-price').val('');
        $('#occupancy-multiplier').val('');
    }
}

$(document).ready(function () {
    // connect to the server through socket.io
    var socket = io();
    setupUpdateEvent(socket);

    // prevent negative signs in positive values fields
    $('.positive').keydown(function (event) {
        if(!event.keyCode == 190) {
            return false;
        }
    });

    // prevent decimal points in integer valued fields
    $('.integer').keydown(function (event) {
        if(event.keyCode == 190) {
            return false;
        }
    });

    // handle keyboard input in the capacity field
    $('#capacity').on('input', function () {
        if (selected.type != TYPE_LOCATION || !locationData[selected.id]) {
            return;
        }
        var value = parseInt($(this).val());
        if (!value) {
            // not  valid number
            value = 0;
        }
        locationData[selected.id].capacity = value;
        socket.emit('set-data', {
            zoneData: zoneData,
            locationData: locationData
        });
    });

    // handle keyboard input in the empty price field
    $('#band1-price').on('input', function () {
        if (selected.type != TYPE_ZONE || !zoneData[selected.id]) {
            return;
        }
        var value = parseFloat(parseFloat($(this).val()).toFixed(2));

        if (!value) {
            // not  valid number, return since the number may not be complete
            return;
        }
        else if (value < 0) {
            // reset negative values
            value = 0;
        }
        zoneData[selected.id].band1Price = value;
        socket.emit('set-data', {
            zoneData: zoneData,
            locationData: locationData
        });
    });

    // handle keyboard input in the full price field
    $('#band6-price').on('input', function () {
        if (selected.type != TYPE_ZONE || !zoneData[selected.id]) {
            return;
        }
        var value = parseFloat(parseFloat($(this).val()).toFixed(2));
        if (!value) {
            // not  valid number, return since the number may not be complete
            return;
        }
        else if (value < 0) {
            // reset negative values
            value = 0;
        }
        zoneData[selected.id].band6Price = value;
        socket.emit('set-data', {
            zoneData: zoneData,
            locationData: locationData
        });
    });

    // handle keyboard input in the full price field
    $('#occupancy-multiplier').on('input', function () {
        if (selected.type != TYPE_ZONE || !zoneData[selected.id]) {
            return;
        }
        var value = parseFloat(parseFloat($(this).val()).toFixed(2));
        if (!value) {
            // not  valid number, return since the number may not be complete
            return;
        }
        else if (value < 0) {
            // reset negative values
            value = 0;
        }
        console.log(value);
        zoneData[selected.id].occupancyMultiplier = value;
        socket.emit('set-data', {
            zoneData: zoneData,
            locationData: locationData
        });
    });

    // update elements when they become out of focus (this replaces any empty fields with their current values)
    $('.focus-update').focusout(function () {
        // refresh currently displayed information
        if (selected.id) {
            displayData(selected.id, selected.type, true);
        }
    });

    // handle a change of value in the show pay and display checkbox
    var showPdCheckbox = $('#show-pd');
    showPdCheckbox.change(function() {
        // reset the map style
        showPd = $(this).is(":checked");
        map.data.setStyle(setStyle);
    });

    // handle when a user changes the highlighting checkbox
    $('#keep-highlight').change(function () {
        // reset the map style
        highlightWhenZoomed = $(this).is(':checked');
        map.data.setStyle(setStyle);
    });

    // handle mouse events to determine if the mouse is pressed
    $(document).mousedown(function() {
        mouseDown = true;
    });
    $(document).mouseup(function() {
        mouseDown = false;
    });

    // set the initial value of whether or not to show pay and display machines
    showPd = showPdCheckbox.is(":checked");
});