/**
 * Created by Connor Weeks on 4/6/2017.
 */

var NUM_INDENT_SPACES = 2;
var MAX_CAPACITY_OCCUPANCY = 10000;
var apiKey = 'AIzaSyAUNGjsvr18E9Cs4Wn7Uk804Y_oR1Soxeo';
var polylineSets = [[]];
var locations = [];

function displayData(id, type) {
    var data = getData(id, type);
    if (data) {
        // determine whether or not the data has location values
        var isLocation = data.locationId != 'None';
        $('#zone-label').text('Zone: ' + data.zoneName);
        $('#location-label').text('Location: ' + data.locationName);
        $('#current-occupancy').val(data.currentOccupancy);
        $('#capacity').val(data.capacity)

    } else {
        $('#zone-label').text('Zone: None');
        $('#location-label').text('Location: None');
        $('#current-occupancy').val('');
        $('#capacity').val('');
    }
}

function updateDataDisplay() {
    if (!locations.length) {
        return;
    }
    $('#most-recent-data').text(JSON.stringify(locations[locations.length - 1], null, NUM_INDENT_SPACES));
    $('#all-data').text(
        locations.map(function (location) {
            return JSON.stringify(location, null, NUM_INDENT_SPACES);
        }).join(',\n')
    );
}

function addSnappedPolyline(data) {
    // accumulate points from the response
    var snappedCoordinates = [];
    for (var i = 0; i < data.snappedPoints.length; i++) {
        var latlng = new google.maps.LatLng(
            data.snappedPoints[i].location.latitude,
            data.snappedPoints[i].location.longitude
        );
        snappedCoordinates.push(latlng);
    }

    // draw polyline and add to set
    var snappedPolyline = new google.maps.Polyline({
        path: snappedCoordinates,
        strokeColor: 'blue',
        strokeWeight: 3
    });

    snappedPolyline.setMap(map);
    // add the newest polyline to the end of the newest set of polylines
    polylineSets[polylineSets.length - 1].push(snappedPolyline);

    // enable the add task button
    $('#show-add-path-btn').prop('disabled', false);
}

/**
 * Returns an array of coordinates in a polyline.
 * @param polyline The polyline object to find the coordinates of.
 * @returns Array of [lng, lat] coordinates.
 */
function getCoordinates(polyline) {
    var pathArray = polyline.getPath().getArray();
    return pathArray.map(function (element) {
        return [element.lng(), element.lat()];
    })
}

/**
 * Returns the number of parking locations in a zone.
 * @param zoneId The ID of the zone to use.
 */
function getNumLocations(zoneId) {
    var count = 0;
    for (var locationId in locationData) {
        if (locationData.hasOwnProperty(locationId) && locationData[locationId].zoneId == zoneId) {
            count++;
        }
    }
    return count;
}
/**
 * File specific map setup function.  This adds a polyline drawing tool
 * to allow the user to add new parking locations.
 */
function setupMap() {
    // create a polyline drawing tool
    var drawingManager = new google.maps.drawing.DrawingManager({
        //drawingMode: google.maps.drawing.OverlayType.POLYLINE,
        drawingControl: true,
        drawingControlOptions: {
            position: google.maps.ControlPosition.TOP_CENTER,
            drawingModes: [
                google.maps.drawing.OverlayType.POLYLINE
            ]
        },
        polylineOptions: {
            strokeColor: 'blue',
            strokeWeight: 2,
            strokeOpacity: 0.3
        }
    });
    drawingManager.setMap(map);

    // Snap-to-road when the polyline is completed.
    drawingManager.addListener('polylinecomplete', function (polyline) {
        // accumulate coordinates for sending
        var path = polyline.getPath();
        var pathValues = [];
        for (var i = 0; i < path.getLength(); i++) {
            pathValues.push(path.getAt(i).toUrlValue());
        }

        // request the path be snapped to roads
        $.get('https://roads.googleapis.com/v1/snapToRoads', {
                interpolate: true,
                key: apiKey,
                path: pathValues.join('|')
            },
            addSnappedPolyline
        );
    });
}


// show the add path modal when the user presses shift + n
$(document).keydown(function (event) {
    var showModalKey = 78; // 'n' key
    if (!$('#show-add-path-btn').prop('disabled') && event.shiftKey && event.which == showModalKey) {
        $('#add-path-modal').modal('show');
    }
});


$(document).ready(function () {

    // connect to the server through socket.io
    var socket = io();
    setupUpdateEvent(socket);

    // focus on the first field when the modal is shown
    $('#add-path-modal').on('shown.bs.modal', function () {
        $('#location-number').focus();
        $('#parking-zones').val(selectedZone);
    });

    // show the add path modal when the user clicks on the add path button
    $('#show-add-path-btn').click(function () {
        $('#add-path-modal').modal('show');
    });

    // handle randomization
    $('#random').click(function () {
        socket.emit('random');
    });

    $('#add-path-btn').click(function () {
        // prevent form submission
        event.preventDefault();

        var locationNumberInput = $('#location-number');
        var locationNameInput = $('#location-name');
        var locationNumber = parseInt(locationNumberInput.val());
        var locationName = locationNameInput.val();
        var zoneId = $('#parking-zones').val();

        // return if the user hasn't enetred in everything
        if (!locationNameInput || !locationName || !zoneId) {
            return;
        }

        var polylines = polylineSets[polylineSets.length - 1];
        var numPolylines = polylines.length;
        var location = {
            type: 'Feature',
            properties: {
                type: 'location',
                locationId: locationNumber,
                zoneId: zoneId,
                name: locationName
            }
        };

        if (numPolylines > 1) {
            location.geometry = {
                type: 'MultiLineString',
                coordinates: polylines.map(getCoordinates)
            };
        } else if (numPolylines == 1) {
            location.geometry = {
                type: 'LineString',
                coordinates: getCoordinates(polylines[0])
            };
        } else {
            return;
        }

        // add the new location and update the data display
        locations.push(location);
        polylineSets.push([]);
        updateDataDisplay();

        // hide the modal, clear the current values, and set the add path button to disabled
        $('#add-path-modal').modal('hide');
        locationNumberInput.val('');
        locationNameInput.val('');
        $('#show-add-path-btn').prop('disabled', true);
    });

    // handle when a user changes the capacity of a zone or location
    $('#capacity').on('input', function () {
        if (!selected.id) {
            return;
        }
        var value = parseInt($(this).val());
        if (!value) {
            // not  valid number
            return;
        }
        value = Math.min(value, MAX_CAPACITY_OCCUPANCY);

        if (selected.type == TYPE_LOCATION && locationData[selected.id]) {
            // this is the easy part, just do as we did in admin mode
            locationData[selected.id].capacity = value;
            socket.emit('set-data', {
                zoneData: zoneData,
                locationData: locationData
            });
        } else if (selected.type == TYPE_ZONE && zoneData[selected.id] && value >= getNumLocations(selected.id)) {
            // determine how much this new capacity differs from the current
            var difference = value - zoneData[selected.id].capacity;
            socket.emit('edit-capacity', {
                zoneId: selected.id,
                difference: difference
            });
        }
    });

    // handle when a user changes the current occupancy of a zone or location
    $('#current-occupancy').on('input', function () {
        if (!selected.id) {
            return;
        }
        var value = parseInt($(this).val());
        // if (!value) {
        //     // not  valid number
        //     value = 0;
        // }
        if (value !== value) {
            // value is NaN
            return;
        }
        value = Math.min(value, MAX_CAPACITY_OCCUPANCY);

        if (selected.type == TYPE_LOCATION && locationData[selected.id]) {
            // this is the easy part, just do as we did in admin mode
            locationData[selected.id].currentOccupancy = value;
            socket.emit('set-data', {
                zoneData: zoneData,
                locationData: locationData
            });
        } else if (selected.type == TYPE_ZONE && zoneData[selected.id]) {
            // determine how much this new occupancy differs from the current
            var difference = value - zoneData[selected.id].currentOccupancy;
            socket.emit('edit-occupancy', {
                zoneId: selected.id,
                difference: difference
            });
        }
    });


    // handle a change of value in the show pay and display checkbox
    var showPdCheckbox = $('#show-pd');
    showPdCheckbox.change(function() {
        // reset the map style
        showPd = $(this).is(":checked");
        map.data.setStyle(setStyle);
    });

    // update elements when they become out of focus (this replaces any empty fields with their current values)
    $('.focus-update').focusout(function () {
        // refresh currently displayed information
        if (selected.id) {
            displayData(selected.id, selected.type);
        }
    });

    // handle when a user changes the highlighting checkbox
    $('#keep-highlight').change(function () {
        // reset the map style
        highlightWhenZoomed = $(this).is(':checked');
        map.data.setStyle(setStyle);
    });

    // set the initial value of whether or not to show pay and display machines
    showPd = showPdCheckbox.is(":checked");
});